@@ -26,13 +26,14 @@
 #include <QWidget>
 #include <QDebug>
 
+#include "debug.h"
 #include <assert.h>
 
 using namespace KXMLGUI;
 
 void ActionList::plug(QWidget *container, int index) const
 {
-    QAction *before = 0L; // Insert after end of widget's current actions (default).
+    QAction *before = nullptr; // Insert after end of widget's current actions (default).
 
     if ((index < 0) || (index > container->actions().count())) {
         qWarning() << "Index " << index << " is not within range (0 - " << container->actions().count();
@@ -46,15 +47,6 @@ void ActionList::plug(QWidget *container, int index) const
     }
 }
 
-void ActionList::unplug(QWidget *container) const
-{
-    Q_FOREACH (QAction *action, *this) {
-        if (container->actions().contains(action)) {
-            container->removeAction(action);
-        }
-    }
-}
-
 ContainerNode::ContainerNode(QWidget *_container, const QString &_tagName,
                              const QString &_name, ContainerNode *_parent,
                              KXMLGUIClient *_client, KXMLGUIBuilder *_builder,
@@ -79,47 +71,25 @@ ContainerNode::~ContainerNode()
 
 void ContainerNode::removeChild(ContainerNode *child)
 {
-    MergingIndexList::Iterator mergingIt = findIndex(child->mergingName);
-    adjustMergingIndices(-1, mergingIt);
     children.removeAll(child);
-    delete child;
+    deleteChild(child);
 }
 
-void ContainerNode::removeChild(QMutableListIterator<ContainerNode *> &childIterator)
+void ContainerNode::deleteChild(ContainerNode *child)
 {
-    MergingIndexList::Iterator mergingIt = findIndex(childIterator.peekNext()->mergingName);
-    adjustMergingIndices(-1, mergingIt);
-    delete childIterator.next();
-    childIterator.remove();
+    MergingIndexList::iterator mergingIt = findIndex(child->mergingName);
+    adjustMergingIndices(-1, mergingIt, QString());
+    delete child;
 }
 
 /*
  * Find a merging index with the given name. Used to find an index defined by <Merge name="blah"/>
  * or by a <DefineGroup name="foo" /> tag.
  */
-MergingIndexList::Iterator ContainerNode::findIndex(const QString &name)
+MergingIndexList::iterator ContainerNode::findIndex(const QString &name)
 {
-    MergingIndexList::Iterator it(mergingIndices.begin());
-    MergingIndexList::Iterator end(mergingIndices.end());
-    for (; it != end; ++it)
-        if ((*it).mergingName == name) {
-            return it;
-        }
-    return it;
-}
-
-/*
- * Check if the given container widget is a child of this node and return the node structure
- * if fonud.
- */
-ContainerNode *ContainerNode::findContainerNode(QWidget *container)
-{
-    Q_FOREACH (ContainerNode *child, children)
-        if (child->container == container) {
-            return child;
-        }
-
-    return 0L;
+    return std::find_if(mergingIndices.begin(), mergingIndices.end(),
+                        [&name](const MergingIndex &idx) { return idx.mergingName == name; });
 }
 
 /*
@@ -153,7 +123,7 @@ ContainerNode *ContainerNode::findContainer(const QString &name, const QString &
         const QList<QWidget *> *excludeList,
         KXMLGUIClient * /*currClient*/)
 {
-    ContainerNode *res = 0L;
+    ContainerNode *res = nullptr;
     ContainerNodeList::ConstIterator nIt = children.constBegin();
 
     if (!name.isEmpty()) {
@@ -197,7 +167,7 @@ ContainerNode *ContainerNode::findContainer(const QString &name, const QString &
 
 ContainerClient *ContainerNode::findChildContainerClient(KXMLGUIClient *currentGUIClient,
         const QString &groupName,
-        const MergingIndexList::Iterator &mergingIdx)
+        const MergingIndexList::iterator &mergingIdx)
 {
     if (!clients.isEmpty()) {
         Q_FOREACH (ContainerClient *client, clients)
@@ -227,8 +197,8 @@ ContainerClient *ContainerNode::findChildContainerClient(KXMLGUIClient *currentG
 
 void ContainerNode::plugActionList(BuildState &state)
 {
-    MergingIndexList::Iterator mIt(mergingIndices.begin());
-    MergingIndexList::Iterator mEnd(mergingIndices.end());
+    MergingIndexList::iterator mIt(mergingIndices.begin());
+    MergingIndexList::iterator mEnd(mergingIndices.end());
     for (; mIt != mEnd; ++mIt) {
         plugActionList(state, mIt);
     }
@@ -238,24 +208,18 @@ void ContainerNode::plugActionList(BuildState &state)
     }
 }
 
-void ContainerNode::plugActionList(BuildState &state, const MergingIndexList::Iterator &mergingIdxIt)
+void ContainerNode::plugActionList(BuildState &state, const MergingIndexList::iterator &mergingIdxIt)
 {
     static const QString &tagActionList = QLatin1String("actionlist");
 
-    MergingIndex mergingIdx = *mergingIdxIt;
-
-    QString k(mergingIdx.mergingName);
-
-    if (k.indexOf(tagActionList) == -1) {
-        return;
-    }
-
-    k = k.mid(tagActionList.length());
-
+    const MergingIndex &mergingIdx = *mergingIdxIt;
     if (mergingIdx.clientName != state.clientName) {
         return;
     }
-
+    if (!mergingIdx.mergingName.startsWith(tagActionList)) {
+        return;
+    }
+    const QString k = mergingIdx.mergingName.mid(tagActionList.length());
     if (k != state.actionListName) {
         return;
     }
@@ -268,13 +232,13 @@ void ContainerNode::plugActionList(BuildState &state, const MergingIndexList::It
 
     state.actionList.plug(container, mergingIdx.value);
 
-    adjustMergingIndices(state.actionList.count(), mergingIdxIt);
+    adjustMergingIndices(state.actionList.count(), mergingIdxIt, QString());
 }
 
 void ContainerNode::unplugActionList(BuildState &state)
 {
-    MergingIndexList::Iterator mIt(mergingIndices.begin());
-    MergingIndexList::Iterator mEnd(mergingIndices.end());
+    MergingIndexList::iterator mIt(mergingIndices.begin());
+    MergingIndexList::iterator mEnd(mergingIndices.end());
     for (; mIt != mEnd; ++mIt) {
         unplugActionList(state, mIt);
     }
@@ -284,9 +248,9 @@ void ContainerNode::unplugActionList(BuildState &state)
     }
 }
 
-void ContainerNode::unplugActionList(BuildState &state, const MergingIndexList::Iterator &mergingIdxIt)
+void ContainerNode::unplugActionList(BuildState &state, const MergingIndexList::iterator &mergingIdxIt)
 {
-    static const QString &tagActionList = QLatin1String("actionlist");
+    static const QString &tagActionList = QStringLiteral("actionlist");
 
     MergingIndex mergingIdx = *mergingIdxIt;
 
@@ -315,21 +279,22 @@ void ContainerNode::unplugActionList(BuildState &state, const MergingIndexList::
         return;
     }
 
-    lIt.value().unplug(container);
-
-    adjustMergingIndices(-int(lIt.value().count()), mergingIdxIt);
+    removeActions(lIt.value());
 
     client->actionLists.erase(lIt);
 }
 
 void ContainerNode::adjustMergingIndices(int offset,
-        const MergingIndexList::Iterator &it)
+                                         const MergingIndexList::iterator &it,
+                                         const QString &currentClientName)
 {
-    MergingIndexList::Iterator mergingIt = it;
-    MergingIndexList::Iterator mergingEnd = mergingIndices.end();
+    MergingIndexList::iterator mergingIt = it;
+    MergingIndexList::iterator mergingEnd = mergingIndices.end();
 
     for (; mergingIt != mergingEnd; ++mergingIt) {
-        (*mergingIt).value += offset;
+        if ((*mergingIt).clientName != currentClientName) {
+            (*mergingIt).value += offset;
+        }
     }
 
     index += offset;
@@ -342,51 +307,46 @@ bool ContainerNode::destruct(QDomElement element, BuildState &state)   //krazy:e
     unplugActions(state);
 
     // remove all merging indices the client defined
-    QMutableListIterator<MergingIndex> cmIt = mergingIndices;
+    QMutableVectorIterator<MergingIndex> cmIt = mergingIndices;
     while (cmIt.hasNext())
         if (cmIt.next().clientName == state.clientName) {
             cmIt.remove();
         }
 
     // ### check for merging index count, too?
-    if (clients.count() == 0 && children.count() == 0 && container &&
+    if (clients.isEmpty() && children.isEmpty() && container &&
             client == state.guiClient) {
-        QWidget *parentContainer = 0L;
-
+        QWidget *parentContainer = nullptr;
         if (parent && parent->container) {
             parentContainer = parent->container;
         }
 
-        assert(builder);
-
+        Q_ASSERT(builder);
         builder->removeContainer(container, parentContainer, element, containerAction);
 
-        client = 0L;
-
+        client = nullptr;
         return true;
     }
 
     if (client == state.guiClient) {
-        client = 0L;
+        client = nullptr;
     }
 
     return false;
-
 }
 
 void ContainerNode::destructChildren(const QDomElement &element, BuildState &state)
 {
     QMutableListIterator<ContainerNode *> childIt = children;
     while (childIt.hasNext()) {
-        ContainerNode *childNode = childIt.peekNext();
+        ContainerNode *childNode = childIt.next();
 
         QDomElement childElement = findElementForChild(element, childNode);
 
         // destruct returns true in case the container really got deleted
         if (childNode->destruct(childElement, state)) {
-            removeChild(childIt);
-        } else {
-            childIt.next();
+            deleteChild(childNode);
+            childIt.remove();
         }
     }
 }
@@ -414,79 +374,53 @@ void ContainerNode::unplugActions(BuildState &state)
     }
 
     QMutableListIterator<ContainerClient *> clientIt(clients);
-
-    /*
-        Disabled because it means in KToolBar::saveState isHidden is always true then,
-        which is clearly wrong.
-
-    if ( clients.count() == 1 && clientIt.current()->client == client &&
-         client == state.guiClient )
-        container->hide(); // this container is going to die, that's for sure.
-                           // in this case let's just hide it, which makes the
-                           // destruction faster
-     */
-
-    while (clientIt.hasNext())
+    while (clientIt.hasNext()) {
         //only unplug the actions of the client we want to remove, as the container might be owned
         //by a different client
-        if (clientIt.peekNext()->client == state.guiClient) {
-            unplugClient(clientIt.peekNext());
-            delete clientIt.next();
+        ContainerClient *cClient = clientIt.next();
+        if (cClient->client == state.guiClient) {
+            unplugClient(cClient);
+            delete cClient;
             clientIt.remove();
-        } else {
-            clientIt.next();
         }
+    }
+}
+
+void ContainerNode::removeActions(const QList<QAction *> &actions)
+{
+    for (QAction *action : actions) {
+        const int pos = container->actions().indexOf(action);
+        if (pos != -1) {
+            container->removeAction(action);
+            for (MergingIndex &idx : mergingIndices) {
+                if (idx.value > pos) {
+                    --idx.value;
+                }
+            }
+            --index;
+        }
+    }
 }
 
 void ContainerNode::unplugClient(ContainerClient *client)
 {
     assert(builder);
 
-    // now quickly remove all custom elements (i.e. separators) and unplug all actions
-
-    QList<QAction *>::ConstIterator custIt = client->customElements.constBegin();
-    QList<QAction *>::ConstIterator custEnd = client->customElements.constEnd();
-    for (; custIt != custEnd; ++custIt) {
-        builder->removeCustomElement(container, *custIt);
-    }
-
     KToolBar *bar = qobject_cast<KToolBar *>(container);
     if (bar) {
         bar->removeXMLGUIClient(client->client);
     }
 
-    client->actions.unplug(container);
-
-    // now adjust all merging indices
-
-    MergingIndexList::Iterator mergingIt = findIndex(client->mergingName);
-
-    adjustMergingIndices(- int(client->actions.count()
-                               + client->customElements.count()),
-                         mergingIt);
+    // now quickly remove all custom elements (i.e. separators) and unplug all actions
+    removeActions(client->customElements);
+    removeActions(client->actions);
 
     // unplug all actionslists
 
     ActionListMap::ConstIterator alIt = client->actionLists.constBegin();
     ActionListMap::ConstIterator alEnd = client->actionLists.constEnd();
     for (; alIt != alEnd; ++alIt) {
-        alIt.value().unplug(container);
-
-        // construct the merging index key (i.e. like named merging) , find the
-        // corresponding merging index and adjust all indices
-        QString mergingKey = alIt.key();
-        mergingKey.prepend(QStringLiteral("actionlist"));
-
-        MergingIndexList::Iterator mIt = findIndex(mergingKey);
-        if (mIt == mergingIndices.end()) {
-            continue;
-        }
-
-        adjustMergingIndices(-int(alIt.value().count()), mIt);
-
-        // remove the actionlists' merging index
-        // ### still needed? we clean up below anyway?
-        mergingIndices.erase(mIt);
+        removeActions(alIt.value());
     }
 }
 
@@ -497,28 +431,20 @@ void ContainerNode::reset()
     }
 
     if (client) {
-        client->setFactory(0L);
+        client->setFactory(nullptr);
     }
 }
 
 int ContainerNode::calcMergingIndex(const QString &mergingName,
-                                    MergingIndexList::Iterator &it,
+                                    MergingIndexList::iterator &it,
                                     BuildState &state,
                                     bool ignoreDefaultMergingIndex)
 {
-    MergingIndexList::Iterator mergingIt;
+    const MergingIndexList::iterator mergingIt = findIndex(mergingName.isEmpty() ? state.clientName : mergingName);
+    const MergingIndexList::iterator mergingEnd = mergingIndices.end();
 
-    if (mergingName.isEmpty()) {
-        mergingIt = findIndex(state.clientName);
-    } else {
-        mergingIt = findIndex(mergingName);
-    }
-
-    MergingIndexList::Iterator mergingEnd = mergingIndices.end();
-    it = mergingEnd;
-
-    if ((mergingIt == mergingEnd && state.currentDefaultMergingIt == mergingEnd) ||
-            ignoreDefaultMergingIndex) {
+    if (ignoreDefaultMergingIndex || (mergingIt == mergingEnd && state.currentDefaultMergingIt == mergingEnd)) {
+        it = mergingEnd;
         return index;
     }
 
@@ -531,7 +457,17 @@ int ContainerNode::calcMergingIndex(const QString &mergingName,
     return (*it).value;
 }
 
-int BuildHelper::calcMergingIndex(const QDomElement &element, MergingIndexList::Iterator &it, QString &group)
+void ContainerNode::dump(int offset)
+{
+    QString indent; indent.fill(QLatin1Char(' '), offset);
+    qDebug() << qPrintable(indent) << name << tagName << groupName << mergingName << mergingIndices;
+    Q_FOREACH (ContainerNode *child, children) {
+        child->dump(offset + 2);
+    }
+}
+
+// TODO: return a struct with 3 members rather than 1 ret val + 2 out params
+int BuildHelper::calcMergingIndex(const QDomElement &element, MergingIndexList::iterator &it, QString &group)
 {
     const QLatin1String attrGroup("group");
 
@@ -545,10 +481,13 @@ int BuildHelper::calcMergingIndex(const QDomElement &element, MergingIndexList::
     int idx;
     if (haveGroup) {
         idx = parentNode->calcMergingIndex(group, it, m_state, ignoreDefaultMergingIndex);
-    } else if (m_state.currentClientMergingIt == parentNode->mergingIndices.end()) {
-        idx = parentNode->index;
     } else {
-        idx = (*m_state.currentClientMergingIt).value;
+        it = m_state.currentClientMergingIt;
+        if (it == parentNode->mergingIndices.end()) {
+            idx = parentNode->index;
+        } else {
+            idx = (*it).value;
+        }
     }
 
     return idx;
@@ -613,7 +552,7 @@ void BuildHelper::processActionOrCustomElement(const QDomElement &e, bool isActi
         return;
     }
 
-    MergingIndexList::Iterator it(m_state.currentClientMergingIt);
+    MergingIndexList::iterator it(m_state.currentClientMergingIt);
 
     QString group;
     int idx = calcMergingIndex(e, it, group);
@@ -627,10 +566,9 @@ void BuildHelper::processActionOrCustomElement(const QDomElement &e, bool isActi
         guiElementCreated = processCustomElement(e, idx);
     }
 
-    if (guiElementCreated)
+    if (guiElementCreated) {
         // adjust any following merging indices and the current running index for the container
-    {
-        parentNode->adjustMergingIndices(1, it);
+        parentNode->adjustMergingIndices(1, it, m_state.clientName);
     }
 }
 
@@ -641,14 +579,15 @@ bool BuildHelper::processActionElement(const QDomElement &e, int idx)
     // look up the action and plug it in
     QAction *action = m_state.guiClient->action(e);
 
-    //qDebug(260) << "BuildHelper::processActionElement " << e.attribute( "name" ) << " -> " << action << " (in " << m_state.guiClient->actionCollection() << ")";
     if (!action) {
         return false;
     }
 
-    QAction *before = 0L;
+    //qCDebug(DEBUG_KXMLGUI) << e.attribute(QStringLiteral("name")) << "->" << action << "inserting at idx=" << idx;
+
+    QAction *before = nullptr;
     if (idx >= 0 && idx < parentNode->container->actions().count()) {
-        before = parentNode->container->actions()[idx];
+        before = parentNode->container->actions().at(idx);
     }
 
     parentNode->container->insertAction(before, action);
@@ -676,7 +615,7 @@ void BuildHelper::processStateElement(const QDomElement &element)
 {
     QString stateName = element.attribute(QStringLiteral("name"));
 
-    if (stateName.isNull() || !stateName.length()) {
+    if (stateName.isEmpty()) {
         return;
     }
 
@@ -738,9 +677,8 @@ void BuildHelper::processMergeElement(const QString &tag, const QString &name, c
 
     if (tag == tagDefineGroup) {
         mergingName.prepend(attrGroup);    //avoid possible name clashes by prepending
-    }
-    // "group" to group definitions
-    else if (tag == tagActionList) {
+                                           // "group" to group definitions
+    } else if (tag == tagActionList) {
         mergingName.prepend(tagActionList);
     }
 
@@ -748,7 +686,7 @@ void BuildHelper::processMergeElement(const QString &tag, const QString &name, c
         return;    //do not allow the redefinition of merging indices!
     }
 
-    MergingIndexList::Iterator mIt(parentNode->mergingIndices.end());
+    MergingIndexList::iterator mIt(parentNode->mergingIndices.end());
 
     QString group(e.attribute(attrGroup));
     if (!group.isEmpty()) {
@@ -764,20 +702,19 @@ void BuildHelper::processMergeElement(const QString &tag, const QString &name, c
     newIdx.mergingName = mergingName;
     newIdx.clientName = m_state.clientName;
 
-    // if that merging index is "inside" another one, then append it right after the "parent" .
+    // if that merging index is "inside" another one, then append it right after the "parent".
     if (mIt != parentNode->mergingIndices.end()) {
         parentNode->mergingIndices.insert(++mIt, newIdx);
     } else {
         parentNode->mergingIndices.append(newIdx);
     }
 
-    if (mergingName == defaultMergingName)
-
-    {
+    if (mergingName == defaultMergingName) {
         ignoreDefaultMergingIndex = true;
     }
 
-    // re-calculate the running default and client merging indices.
+    // re-calculate the running default and client merging indices
+    // (especially important in case the QList data got reallocated due to growing!)
     m_state.currentDefaultMergingIt = parentNode->findIndex(defaultMergingName);
     parentNode->calcMergingIndex(QString(), m_state.currentClientMergingIt,
                                  m_state, ignoreDefaultMergingIndex);
@@ -791,7 +728,7 @@ void BuildHelper::processContainerElement(const QDomElement &e, const QString &t
                                    m_state.guiClient);
 
     if (!containerNode) {
-        MergingIndexList::Iterator it(m_state.currentClientMergingIt);
+        MergingIndexList::iterator it(m_state.currentClientMergingIt);
         QString group;
 
         int idx = calcMergingIndex(e, it, group);
@@ -807,9 +744,12 @@ void BuildHelper::processContainerElement(const QDomElement &e, const QString &t
             return;
         }
 
-        parentNode->adjustMergingIndices(1, it);
+        parentNode->adjustMergingIndices(1, it, m_state.clientName);
 
-        assert(!parentNode->findContainerNode(container));
+        // Check that the container widget is not already in parentNode.
+        Q_ASSERT(std::find_if(parentNode->children.constBegin(), parentNode->children.constEnd(),
+                              [container](ContainerNode *child) { return child->container == container; })
+                == parentNode->children.constEnd());
 
         containerList.append(container);
 
@@ -853,7 +793,7 @@ QWidget *BuildHelper::createContainer(QWidget *parent, int index,
                                       const QDomElement &element, QAction *&containerAction,
                                       KXMLGUIBuilder **builder)
 {
-    QWidget *res = 0L;
+    QWidget *res = nullptr;
 
     if (m_state.clientBuilder) {
         res = m_state.clientBuilder->createContainer(parent, index, element, containerAction);
@@ -887,6 +827,11 @@ void BuildState::reset()
     guiClient = 0;
     clientBuilder = 0;
 
-    currentDefaultMergingIt = currentClientMergingIt = MergingIndexList::Iterator();
+    currentDefaultMergingIt = currentClientMergingIt = MergingIndexList::iterator();
 }
 
+QDebug operator<<(QDebug stream, const MergingIndex &mi) {
+    QDebugStateSaver saver(stream);
+    stream.nospace() << "clientName=" << mi.clientName << " mergingName=" << mi.mergingName << " value=" << mi.value;
+    return stream;
+}
